<?php
/**
 * @file
 * Allunel Sklep installation profile.
 */

/**
 * Return a description of the profile for the initial installation screen.
 */
function allunel_sklep_profile_details() {
  return array(
    'name' => 'Allunel Sklep',
    'description' => 'Online shop specifically designed to work in Polish e-commerce environment.',
    'language' => 'pl'
  );
}

/**
 * Return an array of the modules to be enabled when this profile is installed.
 *
 * The following required core modules are always enabled:
 * 'block', 'filter', 'node', 'system', 'user'.
 */
function allunel_sklep_profile_modules() {
  return array(
    // Core required modules always are enabled by default (system)
    
    // Core Drupal modules:
    'help',
    'menu',
    'path',
    'taxonomy',
    'dblog', // DB logging
    'trigger',

    // Ubercart dependencies:
    'ca', // Conditional Actions
    'token',

    // Ubercart modules:
    'uc_store',
    'uc_product',
    'uc_order',
    'uc_catalog',
    'uc_cart',
    'uc_quote',
    'uc_shipping',
    'uc_flatrate',
    'uc_taxes',
    'uc_attribute',

    // Payments
    'uc_bank_transfer',
    'uc_payment_pack',
    'uc_paypal',
//    'platnosci_pl',
//    'uc_dotpay_pl',
//    'uc_allegro',
//    'uc_ceneo',

    // system
    'module_filter',
    'admin_menu',

    // development
    // @TODO: split to seperate task/profile?
    'devel',
    'devel_generate',
    'uc_generate'

  );
}

/**
 * Return a list of tasks that this profile supports.
 *
 * @return
 *   A keyed array of tasks the profile will perform during the final stage. The
 *   keys of the array will be used internally, while the values will be
 *   displayed to the user in the installer task list.
 */
function allunel_sklep_profile_task_list() {
  return array();
}

/**
 * Perform installation tasks for this installation profile.
 */
function allunel_sklep_profile_tasks(&$task, $url) {
  switch ($task) {
    // Perform tasks when the installation form is initially submitted and all
    // the specified modules have been installed.
    case 'profile':
      // import polish translation
      _allunel_import_polish_translation();

      // Setup the page content type.
      _allunel_sklep_setup_page();

      // Toggle the node info display for products.
      $settings = variable_get('theme_settings', array());
      $settings['toggle_node_info_product'] = 0;
      variable_set('theme_settings', $settings);

      // Use a working store dashboard setting.
      variable_set('uc_store_admin_page_display', 4);

      // Turn off the Ubercart footer.
      variable_set('uc_footer_message', 'none');

      // changet uc cataglog english name to polish
      variable_set('uc_catalog_name', st('Catalog'));
      
      // Set Polish defaults
      _allunel_sklep_set_currency();
      _allunel_sklep_set_weight_length_date();

      // Install Acquia Prosper if the necessary module and themes exist.
      _allunel_sklep_install_theme();

      // Update the menu router information.
      menu_rebuild();

      // Import every country available.
      require_once drupal_get_path('module', 'uc_store') .'/uc_store.admin.inc';

      // Get an array of all the files in the countries directory.
      $files = _country_import_list();

      // Unset any countries from the files array that have already been imported.
      $result = db_query("SELECT * FROM {uc_countries} ORDER BY country_name ASC");
      while ($country = db_fetch_object($result)) {
        unset($files[$country->country_id]);
      }

      // Install any country remaining in the files array.
      foreach ($files as $file) {
        uc_country_import($file['file']);
      }

      // Setup a catalog term and basic product.
      $edit = array('vid' => 1, 'name' => t('Products'));
      taxonomy_save_term($edit);

      // Create some instructional and example nodes.
      _allunel_sklep_create_nodes();

      // Setup some default menu items.
      _allunel_sklep_setup_menu_items();

      // Create vendor role
      db_query("INSERT INTO {role} (name) VALUES ('vendor')");      
			
      // Add some basic permissions for anonymous and authenticated users.
      db_query("UPDATE {permission} SET perm = CONCAT(perm, ', view catalog') WHERE pid = 1");
      db_query("UPDATE {permission} SET perm = CONCAT(perm, ', view catalog, view own orders') WHERE pid = 2");

      // install image feature
      allunel_sklep_features_install();

      // generate sample products
      // @TODO: move it to seperate task?
      _allunel_generate_products();

      // FINISH
      // Move to the completion task.
      $task = 'profile-finished';
      break;
  }
}

function allunel_sklep_features_install() {
  $postprocess = array();

  // Loop through all the .pkg.inc feature include files in the profile.
  foreach (allunel_sklep_list_features(_get_current_profile()) as $filename => $file) {
    // If this feature was selected for inclusion in the form...
    require_once $filename;

    $base = substr($file->name, 0, strlen($file->name) - 4);
    $func = $base .'_pkg_info';
    $feature_info = $func();

    // Install all the modules required by the feature.
    foreach ((array) $feature_info['modules'] as $module) {
      drupal_load('module', $module);
      drupal_install_modules(array($module));
    }

    $postprocess[] = $base . '_pkg_postinstall';
  }

  foreach ($postprocess as $process) {
    if (function_exists($process)) {
      $process();
    }
  }
}

/**
 * Return an array of .pkg.inc files in the profile directory keyed by filename.
 */
function allunel_sklep_list_features($profile) {
  return file_scan_directory("profiles/$profile", '.*\.pkg.inc');
}

/**
 * Install a module or array of modules if the modules exist in the site.
 *
 * @param $module
 *   A module name or array of module names to install if the modules exist.
 * @return
 *   Returns TRUE or FALSE to indicate whether a module actually existed; no
 *     return value for multiple installations by array.
 */
function allunel_sklep_install_module($module) {
  // If an array was passed...
  if (is_array($module)) {
    // Loop through the array and install each one.
    foreach ($module as $mod) {
      allunel_sklep_install_module($mod);
    }
  }
  else {
    // Otherwise go ahead and attempt the install.
    if (drupal_get_path('module', $module)) {
      drupal_load('module', $module);
      drupal_install_modules(array($module));
      return TRUE;
    }
    else {
      return FALSE;
    }
  }
}

/**
 * Setup the page content type.
 */
function _allunel_sklep_setup_page() {
  require_once 'modules/comment/comment.module';

  // Insert the page node type into the database.
  $type = array(
    'type' => 'page',
    'name' => st('Page'),
    'module' => 'node',
    'description' => st("A <em>page</em>, similar in form to a <em>story</em>, is a simple method for creating and displaying information that rarely changes, such as an \"About us\" section of a website. By default, a <em>page</em> entry does not allow visitor comments and is not featured on the site's initial home page."),
    'custom' => TRUE,
    'modified' => TRUE,
    'locked' => FALSE,
    'help' => '',
    'min_word_count' => '',
  );

  $type = (object) _node_type_set_defaults($type);
  node_type_save($type);

  // Default page to not be promoted and have comments disabled.
  variable_set('node_options_page', array('status'));
  variable_set('comment_page', COMMENT_NODE_DISABLED);

  // Don't display date and author information for page nodes by default.
  $theme_settings = variable_get('theme_settings', array());
  $theme_settings['toggle_node_info_page'] = FALSE;
  variable_set('theme_settings', $theme_settings);
}

// Install Acquia Prosper if possible.
function _allunel_sklep_install_theme() {
  $themes = array_keys(system_theme_data());

  if (drupal_get_path('module', 'skinr') && in_array('fusion_core', $themes) && in_array('acquia_prosper', $themes)) {
    // First install Skinr.
    allunel_sklep_install_module('skinr');

    // Then install the themes.
    db_query("UPDATE {system} SET status = 1 WHERE type = 'theme' AND (name = '%s' OR name = '%s')", 'fusion_core', 'acquia_prosper');

    // And finally set Acquia Prosper to be the default.
    variable_set('theme_default', 'acquia_prosper');
  }

  // Configure blocks for Acquia Prosper.
  _allunel_sklep_setup_blocks();
}

// Configure the blocks for Acquia Prosper.
function _allunel_sklep_setup_blocks() {
  $blocks = array(
    array('module' => 'menu', 'delta' => 'primary-links', 'weight' => -5, 'region' => 'footer'),
    array('module' => 'user', 'delta' => 0, 'weight' => -4, 'region' => 'sidebar_first', 'pages' => 'cart/checkout*'),
    array('module' => 'user', 'delta' => 1, 'weight' => -3, 'region' => 'sidebar_first', 'pages' => 'cart/checkout*'),
    array('module' => 'uc_cart', 'delta' => 0, 'weight' => -5, 'region' => 'header', 'pages' => 'cart/checkout*'),
    array('module' => 'uc_catalog', 'delta' => 0, 'weight' => -5, 'region' => 'sidebar_first', 'pages' => 'cart/checkout*'),
  );

  foreach ($blocks as $block) {
    $block['theme'] = 'acquia_prosper';
    $block['status'] = 1;
    drupal_write_record('blocks', $block);
  }

  db_query("INSERT INTO {blocks_roles} (module, delta, rid) VALUES ('%s', %d, %d)", 'user', 1, 2);

  $settings = array(
    'block' => array(
      'uc_catalog-0' => array(
        'prosper-general-styles' => 'prosper-gray-rounded-style prosper-rounded-title',
      ),
      'uc_cart-0' => array(
        'grid16-width' => 'grid16-5',
        'fusion-alignment' => 'fusion-right',
        'prosper-general-styles' => 'prosper-shoppingcart-dark',
      ),
      'menu-primary-links' => array(
        'fusion-menu' => 'fusion-inline-menu',
      ),
    ),
  );
  variable_set('skinr_acquia_prosper', $settings);
}

// Install a few primary link menu items by default.
function _allunel_sklep_setup_menu_items() {
  $items = array(
    array('link_path' => '<front>', 'link_title' => t('Home'), 'weight' => 0),
    array('link_path' => 'catalog', 'link_title' => st('Catalog'), 'weight' => 1),
    array('link_path' => 'node/2', 'link_title' => t('About us'), 'weight' => 2),
  );

  foreach ($items as $item) {
    $item += array(
      'mlid' => 0,
      'module' => 'menu',
      'has_children' => 0,
      'options' => array(
        'attributes' => array(
          'title' => '',
        ),
      ),
      'customized' => 1,
      'original_item' => array(
        'link_title' => '',
        'mlid' => 0,
        'plid' => 0,
        'menu_name' => 'primary-links',
        'weight' => 1,
        'link_path' => '',
        'options' => array(),
        'module' => 'menu',
        'expanded' => 0,
        'hidden' => 0,
        'has_children' => 0,
      ),
      'description' => '',
      'expanded' => 0,
      'parent' => 'primary-links:0',
      'hidden' => 0,
      'plid' => 0,
      'menu_name' => 'primary-links',
    );
    menu_link_save($item);
  }
}

// Create an instructions page, an example product, and an about page.
function _allunel_sklep_create_nodes() {
  // Add an instructional page and set it to the front page.
  $node = new stdClass();
  $node->title = 'Witamy w Twoim sklepie!';
  $default_message = '<p>Kieruj się poniższymi krokami aby skonfigurować swój sklep:</p>';
  
  $default_message .= '<ol>';

  $default_message .= '<li><strong>Zkonfiguruj skelp</strong> Ustaw wszystko w <a href="'. url('admin/settings') .'">ustawieniach admina</a> wszystkie aspekty twojej strony.</li>';

  $default_message .= '<li><strong>Włącz dodatkowe funkcje</strong> Next, visit the <a href="@modules">module list</a> and enable features which suit your specific needs. You can find additional modules in the <a href="#">Drupal modules download section</a>.</li>';
  
  $default_message .= '<li><strong>Wybierz skórkę</strong> To change the "look and feel" of your website, visit the <a href="#">themes section</a>. You may choose from one of the included themes or download additional themes from the <a href="#">Drupal themes download section</a>.</li>';

  $default_message .= '<li><strong>Zacznij dodawać produkty</strong> Finally, you can <a href="#">create content</a> for your website. This message will disappear once you have promoted a post to the front page.</li>';

  $default_message .= '</ol>';

  $default_message .= '<p>For more information, please refer to the <a href="#">help section</a>, or the <a href="#">online Drupal handbooks</a>. You may also post at the <a href="#">Drupal forum</a>, or view the wide range of <a href="#">other support options</a> available.</p>';

  $node->body = $default_message;

  $node->type = 'page';
  $node->created = time();
  $node->changed = time();
  $node->status = 1;
  $node->promote = 1;
  $node->sticky = 0;
  $node->format = 2;
  $node->uid = 1;
  $node->language = 'en';
  node_save($node);

  variable_set('site_frontpage', 'node/1');

  $node = new stdClass();
  $node->title = 'About us';
  $node->body = 'This is a sample about us page that you can edit to fill with information about your store.';
  $node->type = 'page';
  $node->created = time();
  $node->changed = time();
  $node->path = 'about';
  $node->status = 1;
  $node->promote = 0;
  $node->sticky = 0;
  $node->format = 1;
  $node->uid = 1;
  $node->language = 'en';
  node_save($node);

  $node = new stdClass();
  $node->title = 'Example Product';
  $node->body = 'This is a simple example product that you can modify or delete. Use it to test checkout for a shippable item.';
  $node->type = 'product';
  $node->created = time();
  $node->changed = time();
  $node->status = 1;
  $node->promote = 0;
  $node->sticky = 0;
  $node->format = 1;
  $node->uid = 1;
  $node->language = 'en';
  $node->model = 'PRODUCT';
  $node->list_price = 15;
  $node->cost = 5;
  $node->sell_price = 10;
  $node->weight = 5;
  $node->weight_units = 'lb';
  $node->default_qty = 1;
  $node->shippable = 1;
  node_save($node);

  taxonomy_node_save($node, array(1));
}

function _get_current_profile() {
  return basename(__FILE__, '.profile');
}

/**
 * Implementation of hook_form_alter().
 *
 * Allows the profile to alter the site-configuration form. This is
 * called through custom invocation, so $form_state is not populated.
 */
function allunel_sklep_form_alter(&$form, $form_state, $form_id) {
  if ($form_id == 'install_configure') {
    drupal_add_js(drupal_get_path('profile', 'allunel_sklep'). '/allunel_sklep.profile.js');

    // Set default for site name field.
    $form['site_information']['site_name']['#default_value'] = 'Allunel';
    $form['site_information']['site_mail']['#default_value'] = 'sklep@example.com';
    
    $form['admin_account']['account']['name']['#default_value'] = 'admin';
    $form['admin_account']['account']['mail']['#default_value'] = 'admin@example.com';
    
    unset($form['server_settings']['update_status_module']['#default_value']);
  }
}

function _allunel_sklep_set_currency() {
  // set country to Poland
  variable_set('uc_store_country', 616);

  // set currencty to Polish New Złoty
  variable_set('uc_currency_code', 'PLN');
  variable_set('uc_currency_sign', ' zł');
  variable_set('uc_currency_thou', ' ');
  variable_set('uc_currency_dec', ',');
  variable_set('uc_currency_prec', '2');
  variable_set('uc_sign_after_amount', 1);
}

function _allunel_sklep_set_weight_length_date() {
  variable_set('uc_weight_unit', 'kg');
  variable_set('uc_length_unit', 'cm');
  variable_set('uc_date_format_default', 'd/m/Y');
}


// generate sample dummy products
function _allunel_generate_products() {
  // if we have product node type
  if(node_get_types('type', 'product')) {
    $form_id = 'devel_generate_content_form';
    $form_state['values']['node_types'] = array('product' => 'dummy text');
    $form_state['values']['num_nodes'] = 60;
    $form_state['values']['add_terms'] = TRUE;
    $form_state['values']['op'] = t('Do it!');
    drupal_execute($form_id, $form_state);
  }
}

function _allunel_import_polish_translation() {
  require_once './includes/locale.inc';

  $files = _allunel_get_all_translations_files();

  // setup default file object
  $file = new stdClass;
  $file->filemime = 'application/octet-stream';
  $file->uid = 0;

  // for all translations
  foreach($files as $f) {
    // update file object
    $file->filename = $f;
    $file->filepath = _allunel_get_filepath($file->filename);

    // try to import translation
    if (_locale_import_po($file, $langcode = 'pl', $mode = 1, $group = 'default') == FALSE) {
      $variables = array('%filename' => $file->filename);
      drupal_set_message(t('The translation import of %filename failed.', $variables), 'error');
      watchdog('locale', 'The translation import of %filename failed.', $variables, WATCHDOG_ERROR);
    }
  }
}

function _allunel_get_filepath($filename) {
  $basedir = dirname(__FILE__);
  $filepath = $basedir. '/translations/'. $filename;
  
  return $filepath;
}

function _allunel_get_all_translations_files() {
  $basedir = dirname(__FILE__);
  if ($handle = opendir($basedir.'/translations')) {
    $files = array();
    /* This is the correct way to loop over the directory.
     * http://php.net/manual/en/function.readdir.php
     */
    while (false !== ($filename = readdir($handle))) {
        if(in_array($filename, array('.', '..', 'pl.po'))) {
          continue;
        }
        $files[] = $filename;
    }
    closedir($handle);

    return $files;
  }

  return FALSE;
}
